﻿#include "UdpSearch.h"
#include <QtNetwork>


// init the static values
UdpSearch       *UdpSearch::INSTANCE             = NULL;
const int        UdpSearch::BROADCAST_PORT       = 6000;
const QByteArray UdpSearch::BROADCAST_KEYWORD    = "WhoisDobotM1";
const int        UdpSearch::DEVICE_LIFE_OVERTIME = 5;
const int        UdpSearch::LOCAL_PORT           = 2046;

UdpSearch *UdpSearch::Instance(QObject *parent)
{
    if (INSTANCE == 0) {
        INSTANCE = new UdpSearch(parent);
    }

    return INSTANCE;
}

UdpSearch::UdpSearch(QObject *parent)
    : QObject(parent)
    , m_socketMap()
    , m_signalMapper(NULL)
    , m_deviceListMutex()
    , m_deviceMap()
{
    // clean up the containers
    m_socketMap.clear();
    m_deviceMap.clear();

    // setup signalMapper
    m_signalMapper = new QSignalMapper(this);
    connect(m_signalMapper, SIGNAL(mapped(QObject *)),
            this, SLOT(ProcessBroadCastDatagrams(QObject *)));

    // search dobot via udp
    SearchDobotDevice(NULL);
}

UdpSearch::~UdpSearch()
{
    // release sockets
    foreach (QUdpSocket *socket, m_socketMap.values()) {
        if (socket) {
            socket->deleteLater();
            socket = NULL;
        }
    }

    // release signalMapper
    if (m_signalMapper) {
        m_signalMapper->deleteLater();
        m_signalMapper = NULL;
    }
}

void UdpSearch::SearchDobotDevice(QStringList *deviceList)
{
    // send broadcast message
    foreach (QString localIP, m_socketMap.keys()) {

        QStringList ipData = localIP.split(".");
        QString broadIP = ipData[0] + "." + ipData[1] + "." + ipData[2] + ".255";
        QUdpSocket *socket = m_socketMap[localIP];

        socket->writeDatagram(BROADCAST_KEYWORD,
                              QHostAddress(broadIP),
                              BROADCAST_PORT);

        //qDebug() << QString("Broadcast：") + broadIP;
    }

    if (deviceList != NULL)
        *deviceList = m_deviceMap.keys();

    SearchNetworkCard();
    DeviceLifeCheck();
}

void UdpSearch::ProcessBroadCastDatagrams(QObject *object)
{
    QUdpSocket *socket =  (QUdpSocket*)object;

    QByteArray   datagram("");
    QHostAddress senderIP("0.0.0.0");

    while (socket->hasPendingDatagrams()) {
        datagram.resize(socket->pendingDatagramSize());

        // read data
        qint64 res = socket->readDatagram(datagram.data(),
                                          datagram.size(),
                                         &senderIP,
                                          NULL);

        if (res != -1) { // success of reading data
            // change QHostAddress to String
            QString targetIP(senderIP.toString());
            // change datagram to String
            QString dataStr(datagram.data());

            // check IP is in the begining of datagram or not
            if (dataStr.contains("dobotM1_") == true) {
                const QString name(dataStr.mid(8));
                QString device("");
                if(name != "") {
                    device = targetIP + "(" + name + ")";
                } else {
                    device = targetIP;
                }
                DeviceListAdd(device, socket->peerAddress().toString());
            }
        }
    }
}

void UdpSearch::DeviceListAdd(QString &device, QString localIP)
{
    //qDebug() << "UDP RecvFrom:" << device;
    m_deviceListMutex.lock();
    if (m_deviceMap.keys().indexOf(device) == -1) {
        m_deviceMap.insert(device, DEV_INFO(QTime::currentTime(), localIP));
    } else {
        m_deviceMap[device].lifeTimeStamp = QTime::currentTime();
    }
    m_deviceListMutex.unlock();
}

void UdpSearch::DeviceLifeCheck()
{
    QTime       currentTime(QTime::currentTime());  // current time stamp
    QStringList deviceList(m_deviceMap.keys());     // device list of map

    foreach (QString device, deviceList) {
        if (m_deviceMap[device].lifeTimeStamp.secsTo(currentTime) >
            DEVICE_LIFE_OVERTIME) {

            // lost device feeback over life overtime
            m_deviceListMutex.lock();
            m_deviceMap.remove(device);
            m_deviceListMutex.unlock();
        }
    }
}

QString UdpSearch::GetLocalIP(QString &device)
{
    return m_deviceMap[device].localIP;
}

void UdpSearch::SearchNetworkCard()
{;
    QHostInfo           info(QHostInfo::fromName(QHostInfo::localHostName()));
    QList<QHostAddress> addressList(info.addresses());
    QStringList         localIPList(m_socketMap.keys());

    // check old network card is living or not
    foreach(QString localIP, localIPList) {
        bool localIPLiving(false);
        foreach(QHostAddress address, addressList) {
            if (address.toString() == localIP) {
                localIPLiving = true;
                break;
            }
        }
        if (!localIPLiving) {
            // lost network card, release its socket
            if (m_socketMap[localIP] != NULL) {
                m_socketMap[localIP]->deleteLater();
                m_socketMap[localIP] = NULL;
            }

            m_socketMap.remove(localIP);
        }
    }

    // check new network
    int portTemp = LOCAL_PORT;
    foreach(QHostAddress address, addressList) {
        if (address.protocol() != QAbstractSocket::IPv4Protocol) {
            continue;
        }

        bool isNewLocalIP(true);
        foreach (QString localIP, localIPList) {
            if (localIP == address.toString()) {
                isNewLocalIP = false;
                break;
            }
        }

        if (isNewLocalIP) {
            // new network card
            QUdpSocket *socket = new QUdpSocket(this);

            while(1){
                bool res = socket->bind(address,
                             portTemp,
                             QUdpSocket::DontShareAddress);
                if (res) {
                    //qDebug() << "bind success!" << portTemp;
                    break;
                }
                else {
                    //qDebug() << "bind failed!" << portTemp;
                    ++portTemp;
                }

                if (portTemp > 49151) {
                    break;
                }

            }

            m_socketMap.insert(address.toString(), socket);

            m_signalMapper->setMapping(socket, (QObject*)socket);

            connect(socket, SIGNAL(readyRead()),
                    m_signalMapper, SLOT(map()));
        }
    }
}

void UdpSearch::Pacemaker(QString device)
{
    m_deviceListMutex.lock();
    if (m_deviceMap.keys().indexOf(device) != -1) {
        m_deviceMap[device].lifeTimeStamp = QTime::currentTime();
    }
    m_deviceListMutex.unlock();
}
